#ifndef __RT_PROTO_QOSPF__H__
#define __RT_PROTO_QOSPF__H__

#include "packet.h"
#include "agent.h"
#include "qospf.h"
#include "ip.h"

#include <iostream>

class rtProtoQOSPF : public Agent, public QOSPF::LsNode  {
 public:
  rtProtoQOSPF() : Agent(PT_RTPROTO_QOSPF) { 
    
  }
  virtual int command(int argc, const char* const* argv);
  bool sendMessage(int destId, u_int32_t messageId, int size);
  void receiveMessage(int sender, u_int32_t msgId);
  
  int getNodeId() { return nodeId_; }
  QOSPF::LsLinkStateList* getLinkStateListPtr()  { return &linkStateList_; }
  QOSPF::LsNodeIdList* getPeerIdListPtr() { return &peerIdList_; }
  QOSPF::LsDelayMap* getDelayMapPtr() { 
    return delayMap_.empty() ? (QOSPF::LsDelayMap *)NULL : &delayMap_;
  }
  void recv(Packet* p, Handler*);

 protected:
  void initialize();
  void setDelay(int nbrId, double delay) {
    delayMap_.insert(nbrId, delay);
  }
  void intfChanged();
  void computeQOSRoutes() { routing_.computeQOSRoutes(); }
  void sendBufferedMessages() { routing_.sendBufferedMessages(); }
  void sendUpdates() { routing_.sendLinkStates(); }

 private:
  typedef QOSPF::LsMap<int, ns_addr_t> PeerAddrMap; // addr for peer Id
  PeerAddrMap peerAddrMap_;
  int nodeId_;
  int LS_ready_;	// to differentiate fake and real LS, debug, 0 == no
			// needed in recv and sendMessage;
  
  QOSPF::LsLinkStateList linkStateList_;
  QOSPF::LsNodeIdList peerIdList_;
  QOSPF::LsDelayMap delayMap_;
  QOSPF::QOSRouting routing_;
  
  int findPeerNodeId(ns_addr_t agentAddr);
  
};

#endif 
